@@ -1,4 +1,4 @@
-/* $Id: pcpserver.c,v 1.42 2015/11/05 10:59:34 nanard Exp $ */
+/* $Id: pcpserver.c,v 1.44 2016/01/19 10:03:29 nanard Exp $ */
 /* MiniUPnP project
  * Website : http://miniupnp.free.fr/
  * Author : Peter Tatrai
@@ -854,7 +854,7 @@ static void DeletePCPPeer(pcp_info_t *pcp_msg_info)
 					    rhost2, sizeof(rhost2), &rport2,
 					    iaddr2, sizeof(iaddr2), &iport2,
 					    &proto2, desc, sizeof(desc),
-					    &timestamp, NULL) > 0)
+					    &timestamp, NULL) >= 0)
 #endif /* 0 */
 		     ;
 	     index++)
@@ -989,17 +989,34 @@ static int CreatePCPMap_FW(pcp_info_t *pcp_msg_info)
 {
 #ifdef ENABLE_UPNPPINHOLE
 	int uid;
-	int r = upnp_add_inboundpinhole(NULL, 0,
+	int r;
+	/* first check if pinhole already exists */
+	uid = upnp_find_inboundpinhole(NULL, 0,
 					pcp_msg_info->mapped_str,
 					pcp_msg_info->int_port,
 					pcp_msg_info->protocol,
-					pcp_msg_info->desc,
-					pcp_msg_info->lifetime,
-					&uid);
-	if (r < 0)
-		return PCP_ERR_NO_RESOURCES;
-	pcp_msg_info->ext_port = pcp_msg_info->int_port;
-	return PCP_SUCCESS;
+					NULL, 0, /* desc */
+					NULL /* lifetime */);
+	if(uid >= 0) {
+		/* pinhole already exists, updating */
+		syslog(LOG_INFO, "updating pinhole to %s:%hu %s",
+		       pcp_msg_info->mapped_str, pcp_msg_info->int_port,
+		       (pcp_msg_info->protocol == IPPROTO_TCP)?"TCP":"UDP");
+		r = upnp_update_inboundpinhole((unsigned short)uid, pcp_msg_info->lifetime);
+		return r >= 0 ? PCP_SUCCESS : PCP_ERR_NO_RESOURCES;
+	} else {
+		r = upnp_add_inboundpinhole(NULL, 0,
+						pcp_msg_info->mapped_str,
+						pcp_msg_info->int_port,
+						pcp_msg_info->protocol,
+						pcp_msg_info->desc,
+						pcp_msg_info->lifetime,
+						&uid);
+		if (r < 0)
+			return PCP_ERR_NO_RESOURCES;
+		pcp_msg_info->ext_port = pcp_msg_info->int_port;
+		return PCP_SUCCESS;
+	}
 #else
 	UNUSED(pcp_msg_info);
 	return PCP_ERR_NO_RESOURCES;
@@ -1063,7 +1080,7 @@ static void DeletePCPMap(pcp_info_t *pcp_msg_info)
 	/* remove all the mappings for this client */
 	int index;
 	unsigned short eport2, iport2;
-	char iaddr2[16];
+	char iaddr2[INET6_ADDRSTRLEN];
 	int proto2;
 	char desc[64];
 	unsigned int timestamp;
@@ -1087,7 +1104,7 @@ static void DeletePCPMap(pcp_info_t *pcp_msg_info)
 				    NULL, 0, NULL,
 				    iaddr2, sizeof(iaddr2), &iport2,
 				    &proto2, desc, sizeof(desc),
-				    &timestamp, NULL) > 0)
+				    &timestamp, NULL) >= 0)
 #endif /* ENABLE_UPNPPINHOLE */
 		     ;
 	     index++)
